#include "C:\.......\Main.h"
#include <LCD.C>

//-----------------------------------------------------------------------------
// Parameter
//

// ds1822 scratchpad registers
#define DS1822_SP_TLSB  0
#define DS1822_SP_TMSB  1
#define DS1822_SP_HLIM  2
#define DS1822_SP_LLIM  3
#define DS1822_SP_CFG   4
#define DS1822_SP_RES0  5
#define DS1822_SP_RES1  6
#define DS1822_SP_RES2  7
#define DS1822_SP_CRC   8

// ds1822 rom registers
#define DS1822_ROM_DEVTYPE  0
#define DS1822_ROM_SERIAL1  1
#define DS1822_ROM_SERIAL2  2
#define DS1822_ROM_SERIAL3  3
#define DS1822_ROM_SERIAL4  4
#define DS1822_ROM_SERIAL5  5
#define DS1822_ROM_SERIAL6  6
#define DS1822_ROM_CRC      7

// ds1822 command set
#define DS1822_CMD_READROM           0x33
#define DS1822_CMD_SKIPROM           0xCC
#define DS1822_CMD_CONVERTTEMP       0x44
#define DS1822_CMD_WRITESCRATCHPAD   0x4E
#define DS1822_CMD_READSCRATCHPAD    0xBE
#define DS1822_CMD_COPYSCRATCHPAD    0x48


int read_ok=0;

//-----------------------------------------------------------------------------
// Global Interupt enable/disable
//
void onewire_disable_interrupts(int disable)
{
   if (disable)
     disable_interrupts(GLOBAL);
   else
     enable_interrupts(GLOBAL);
}


//-----------------------------------------------------------------------------
// Sensor 1
//

//-----------------------------------------------------------------------------
// Generate a 1-Wire reset, return 1 if no presence detect was found,
// return 0 otherwise.
//
int OWTouchReset(void)
{
int result;
onewire_disable_interrupts(TRUE);
delay_us(0);
output_low(PIN_A0); // Drives DQ low
delay_us(480);
output_float(PIN_A0); // Releases the bus
delay_us(70);
result = input(PIN_A0) & 0x01; // wenn slave auf "0" zieht, dann result = 0
delay_us(410); // Complete the reset sequence recovery
onewire_disable_interrupts(FALSE);
return result; // Return sample presence pulse result
}
//-----------------------------------------------------------------------------
// Send a 1-Wire write bit. Provide 10us recovery time.
//
void OWWriteBit(int bit)
{
if (bit)
{
// Write '1' bit
output_low(PIN_A0); // Drives DQ low
delay_us(6);
output_float(PIN_A0); // Releases the bus
delay_us(64); // Complete the time slot and 10us recovery
}
else
{
// Write '0' bit
output_low(PIN_A0); // Drives DQ low
delay_us(60);
output_float(PIN_A0); // Releases the bus
delay_us(10);
}
}
//-----------------------------------------------------------------------------
// Read a bit from the 1-Wire bus and return it. Provide 10us recovery time.
//
int OWReadBit(void)
{
int result;
output_low(PIN_A0); // Drives DQ low
delay_us(6);
output_float(PIN_A0); // Releases the bus
delay_us(9);
result = input(PIN_A0) & 0x01; // Sample the bit value from the slave
delay_us(55); // Complete the time slot and 10us recovery
return result;
}

//-----------------------------------------------------------------------------
// Write 1-Wire data byte
//
void OWWriteByte(int data)
{
int loop;
onewire_disable_interrupts(TRUE);
// Loop to write each bit in the byte, LS-bit first
for (loop = 0; loop < 8; loop++)
{
OWWriteBit(data & 0x01);
// shift the data byte for the next bit
data >>= 1;
}
onewire_disable_interrupts(FALSE);
}
//-----------------------------------------------------------------------------
// Read 1-Wire data byte and return it
//
int OWReadByte(void)
{
int loop, result=0;

onewire_disable_interrupts(TRUE);
for (loop = 0; loop < 8; loop++)
{
// shift the result to get it ready for the next bit
result >>= 1;
// if result is one, then set MS bit
if (OWReadBit())
result |= 0x80;
}
onewire_disable_interrupts(FALSE);
return result;
}
//-----------------------------------------------------------------------------
// Write a 1-Wire data byte and return the sampled result.
//
int OWTouchByte(int data)
{
int loop, result=0;
for (loop = 0; loop < 8; loop++)
{
// shift the result to get it ready for the next bit
result >>= 1;
// If sending a '1' then read a bit else write a '0'
if (data & 0x01)
{
if (OWReadBit())
result |= 0x80;
}
else
OWWriteBit(0);
// shift the data byte for the next bit
data >>= 1;
}
return result;
}

//----------------------------------------------------------------------------
// Speicher des Temperatursensors auslesen
//
int read_scratchpad()
{
   int data[9];

   if (owTouchReset())              // Reset the 1-Wire bus
   {                                // wenn result = 1 --> keine Slave Prsenz
      lcd_putc("\f");
      printf(lcd_putc,"kein Sensor 1!");
      delay_ms(1000);              // 1s warten
      return 0;                    // 0 wenn kein Slave gefunden
   }
   
   OWWriteByte(DS1822_CMD_SKIPROM);
   OWWriteByte(DS1822_CMD_CONVERTTEMP);

   delay_ms(760);                // "worst case time" fr Temperaturumwandlung
   
   if (owTouchReset())              // Reset the 1-Wire bus
   {
      lcd_putc("\f");               // wenn result = 1 --> keine Slave Prsenz
      printf(lcd_putc,"kein Sensor 1!");
      delay_ms(1000);              // 1s warten
      return 0;                    // 0 wenn kein Slave gefunden
   }
   
   OWWriteByte(DS1822_CMD_SKIPROM);
   OWWriteByte(DS1822_CMD_READSCRATCHPAD);

   data[DS1822_SP_TLSB]=OWReadByte();    // 0 Tlsb
   data[DS1822_SP_TMSB]=OWReadByte();    // 1 Tmsb
   data[DS1822_SP_HLIM]=OWReadByte();    // 2 Thlim
   data[DS1822_SP_LLIM]=OWReadByte();    // 3 Tllim
   data[DS1822_SP_CFG]=OWReadByte();     // 4 Config
   data[DS1822_SP_RES0]=OWReadByte();    // 5 RES0
   data[DS1822_SP_RES1]=OWReadByte();    // 6 RES1
   data[DS1822_SP_RES2]=OWReadByte();    // 7 RES2
   data[DS1822_SP_CRC]=OWReadByte();     // 8 CRC
   
   read_ok=1;

}


//-----------------------------------------------------------------------------
// Hauptprogramm
//
void main(void)
{
   int Temp_ist=0;

   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_OFF);
   setup_psp(PSP_DISABLED);
   setup_spi(FALSE);
   setup_wdt(WDT_OFF);
   setup_timer_0(RTCC_INTERNAL);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DIV_BY_1,255,1);
   setup_timer_3(T3_DISABLED|T3_DIV_BY_1);
   setup_comparator(NC_NC_NC_NC);

  lcd_init();
  delay_ms(500);
  lcd_putc("\f");

  printf(lcd_putc,"Luefterregler");
  lcd_putc("\n");
  printf(lcd_putc,"von .....");
  delay_ms(2000);                             // 2s warten
  lcd_putc("\f");                             // Display lschen

   while (1)
   {
         read_scratchpad();

         if (read_ok=1)
         {
            if (DS1822_SP_TMSB != 0xff)         // wenn MSB nicht FFh --> positive Temperatur
             {
               Temp_ist = DS1822_SP_TLSB / 2;     // Temperatur mit 0,5C Genauigkeit
             }
            else                                // negative Temperatur
            {
               Temp_ist = (DS1822_SP_TLSB ^ 0xff)+1;   // exklusiv oder Verknpfung mit 0xFF
               Temp_ist = Temp_ist / 2;          // Temperatur berechnen mit 0,5C Genauigkeit
            }
   
            // LCD Display Temperaturausgabe
            lcd_putc("\f");
            printf(lcd_putc,"\Temp: %3.1u", Temp_ist);
            delay_ms(1000);
            read_ok=0;         
         }
   }
}
